#!/usr/bin/perl -w
# classebull07.pl
# Auteur: Jean-Marie Pacquet
#
# Pour une date donne (option "-d aaaa-mm-jj") il calcule:
# 1. le rendement de chaque portefeuille pour chaque jour de cotation
# 2. l'cart-type  la date demande
# 3. le ratio de Sharpe  la date demande
# il classe ensuite les portefeuilles pour chacun de ces trois critres
# il transfre via FTP (option "-f") les fichiers rsultants sur le serveur
#
use strict;
use Getopt::Std;
use DBI;
use Net::FTP;

#
###############################################################################
#
my $urlftp = "serveur-perso-ftp";
my $loginftp = "utilisateur-ftp";
my $mdpftp = "mot-de-passe-ftp";
my $loginpg = "utilisateur-sql";
my $mdppg = "mot-de-passe-sql";
# date du classement
my $anc;
my $moisc;
my $jourc;
# Rcuprer les options de la ligne de commande
our($opt_d, $opt_f);
if (getopts('fd:')) {
    print $opt_d . "\n" if ($opt_d);
} else {
    die "classebull: usage: classebull [-d aaaa-mm-jj] [-f]\n";
}
if ($opt_d) {
    $anc = substr($opt_d,0,4);
    $moisc = substr($opt_d,5,2);
    $jourc = substr($opt_d,8,2);
} else {
    $anc = 2007;
    $moisc = 12;
    $jourc = 31;
}
# Ouvrir la base de donnes
#
my $dbh = DBI->connect('DBI:Pg:dbname=thebull',
		       $loginpg,
		       $mdppg,
		       {
			   AutoCommit => 0
		       }
		       ) or die "Erreur de connexion  la base: " . DBI->errstr;
# date de rfrence pour les cotes
my $origine = "2006-12-31";
# date de dernire cotation
my $an1 = "2006";
my $mois1 = "12";
my $jour1 = "31";
# compteur pour les jours de cotation
my $ccount; 
#
# prparer la liste des portefeuilles
#
my $select_joueur = $dbh->prepare('SELECT portif FROM joueurs07');
my $joueur; # rsultat de la requte $select_joueur
my $select_portif = $dbh->prepare('SELECT portif,pseudo FROM joueurs07');
my @joueur; # rsultat de la requte $select_portif
my %portif; # tableau de la valeur des portefeuilles par joueur
my %pseudo; # tableau des pseudos
#
# prparer la liste des valeurs pour un portefeuille
#
my $select_lignes = $dbh->prepare('SELECT valeur FROM lignes07 WHERE portif = ?');
my $valeur; # rsultat de la requte $select_lignes
#
# prparer la slection des valeurs d'un portefeuille trie par rang
#
my $select_vallib = $dbh->prepare('SELECT lignes07.valeur,libelle FROM lignes07 JOIN valeurs07 ON valeurs07.valeur = lignes07.valeur WHERE portif = ? ORDER BY rang');
my @vallib; # rsultat de la requte $select_vallib
#
# prparer la liste des jours de cotation depuis l'origine
#
my $select_jours = $dbh->prepare('SELECT DISTINCT jour FROM cotations07 WHERE jour > ?');
my $date; # rsultat de la requte $select_jours
#
# prparer la liste des cotations  une date donne
#
my $select_cotes = $dbh->prepare('SELECT cote,valeur FROM cotations07 WHERE jour = ?');
my $select_dividende = $dbh->prepare('SELECT dividende,valeur FROM dividendes07 WHERE jour = ?');
my @cote; # tableau de rsultat de la requte $select_cotes
my %cotedorigine; # tableau des cotes d'origine index par valeur
my %cotedujour; # tableau des cotes du jour index par valeur
my %dividende; # tableau des dividendes index par valeur
my @detachement; # tableau de rsultat de la requte $select_dividende
#
# prparer la slection et l'insertion du rsultat
#
my $select_resultat = $dbh->prepare('SELECT resultat FROM resultats07 WHERE portif = ? AND jour = ?');
my $insert_resultat = $dbh->prepare('INSERT INTO resultats07 (portif,jour,resultat) VALUES (?,?,?)');
my $update_resultat = $dbh->prepare('UPDATE resultats07 SET place = ? WHERE portif = ? AND jour = ?');

die "Impossible de prparer les requtes\n"
    unless defined $select_joueur && defined $select_portif
        && defined $select_lignes && defined $select_vallib
        && defined $select_jours
        && defined $select_cotes && defined $select_dividende
        && defined $select_resultat && $insert_resultat && $update_resultat;
#
# initialisation des cotes des valeurs
#    et des dividendes
#
$select_cotes->execute($origine);
$ccount = 0;
while (@cote = $select_cotes->fetchrow_array()) {
    $cotedorigine{$cote[1]} = $cote[0];
    $cotedujour{$cote[1]} = $cote[0];
    $dividende{$cote[1]} = 0;
    $ccount++;
}
if ($ccount > 0) {
    print $ccount . " initialisations de cotes  la date " . $origine . "\n";
}
#
# initialisation de la valeur des portefeuilles,
# du cumul des variations de valeur (cumuldelta)
# et du cumul des carrs de cette variation (cumulcarres)
#
my $pcount = 0;
my %cumuldelta; # cumul des variations de valeur (pour le calcul de la moyenne)
my %cumulcarres; # cumul des carrs des variations de valeur (pour le calcul de la variance)
$select_portif->execute();
while (@joueur = $select_portif->fetchrow_array()) {
    $portif{$joueur[0]} = 100;
    $cumuldelta{$joueur[0]} = 0;
    $cumulcarres{$joueur[0]} = 0;
    $pseudo{$joueur[0]} = "\U$joueur[1]\E";
    $pcount += 1;
}
print $pcount . " initialisations de portefeuilles\n";
#
# boucle de mise  jour du rsultat de chaque portefeuille
# (pour chaque jour de cotation depuis l'origine)
# calcul du cumul des variations de valeur
# et calcul du cumul des carrs de cette variation
#
my $somme;
my $numligne;
my $variation;
my $pourcentage;
my $resultat;
my %rendt; # le rendement
my @classement; # le classement final
my $place;
my $rcount = 0; # nombre de mises  jour de rsultat
my $rstart;
$ccount = 0; # nombre de jours de cotation
$select_jours->execute($origine);
while ($date = $select_jours->fetchrow_array()) {
    print "Date= " . $date . "\n"; # DEBUG
    my $ann = substr($date,0,4);
    my $moisn = substr($date,5,2);
    my $journ = substr($date,8,2);
    if (($anc < $ann) or (($anc == $ann) and ($moisc < $moisn))
	or (($anc == $ann) and ($moisc == $moisn) and ($jourc < $journ))) {
	last;
    }
    # dernire date de cotation
    $an1 = $ann;
    $mois1 = $moisn;
    $jour1 = $journ;
    # mise  jour des dividendes (pour les valeurs dont c'est le jour de dtachement)
    $select_dividende->execute($date);
    while (@detachement = $select_dividende->fetchrow_array()) {
        $dividende{$detachement[1]} += $detachement[0];
    }
    # mise  jour des cotes (pour les valeurs qui ont cot)
    $select_cotes->execute($date);
    while (@cote = $select_cotes->fetchrow_array()) {
	$cotedujour{$cote[1]} = $cote[0];
	if ($dividende{$cote[1]}) {
	    $cotedujour{$cote[1]} = sprintf("%.2f",$cote[0] + $dividende{$cote[1]});
	}
    }
    #
    # Pour chaque joueur (et donc chaque portefeuille)
    #
    $rstart = $rcount;
    $select_joueur->execute();
    while ($joueur = $select_joueur->fetchrow_array()) {
#	print "Joueur= " . $joueur . "\n"; # DEBUG
	# est-ce qu'on a dj le rsultat?
	$select_resultat->execute($joueur,$date);
	if ($select_resultat->rows == 0) {
	    # on ne l'a pas, on le calcule
	    $select_lignes->execute($joueur);
	    $somme = 0;
	    $numligne = 0;
	    # pour chaque ligne du portefeuille
	    while ($valeur = $select_lignes->fetchrow_array()) {
		$variation = 100 * ($cotedujour{$valeur} - $cotedorigine{$valeur}) / $cotedorigine{$valeur};
		$somme += $variation;
		$numligne++;
	    }
	    $select_lignes->finish;
	    $resultat = $somme;
	    if ($numligne > 1) {
		$resultat /= $numligne;
	    }
	    # on l'insre
	    $insert_resultat->execute($joueur,$date,$resultat)
		or die "Impossible d'ajouter un rsultat\n";
	    $dbh->commit();
	    $rcount += 1;
	} else {
	    # on a dj le rsultat, on le rcupre
	    $resultat = $select_resultat->fetchrow_array();
	}
	# variation en pourcentage
	$variation = ($resultat + 100 - $portif{$joueur}) / $portif{$joueur};
	# nouvelle valeur du portefeuille
	$rendt{$joueur} = $resultat;
	$portif{$joueur} = 100 + $resultat;
	# cumul des variations pour ce portefeuille
	$cumuldelta{$joueur} += $variation;
	# cumul des carrs des variations pour ce portefeuille
	$cumulcarres{$joueur} += ($variation * $variation);
    }
    if ($rcount > $rstart) {
	#
	# tri par ordre dcroissant du rendement
	#
	@classement = sort { $rendt{$b} <=> $rendt{$a} } keys %rendt;
	# enregistrement de la place du joueur dans le classement
	$place = 0;
	foreach $joueur (@classement) {
	    if ($joueur < 1000) {
		$place += 1;
	    }
	    $update_resultat->execute($place,$joueur,$date)
		or die "Impossible d'enregistrer la place\n";
	    $dbh->commit();
	}
    }
    # ajustement du nombre de jours de cote
    $ccount += 1;
}
print $rcount . " mises  jour de rsultats\n";
print $ccount . " jours de cote\n";
#
# boucle de calcul final pour chaque joueur
#
my $moyenne; # la moyenne des variations de valeur du portefeuille
my $variance; # la variance (= moyenne des carrs des variations)
my %ecart_type; # l'cart type (= racine carre de la variance)
my %sharpe; # le ratio de Sharpe
my $livreta = .0225; # le taux de rendement sans risque (livret A)
#
$select_joueur->execute();
while ($joueur = $select_joueur->fetchrow_array()) {
    if ($ccount == 0) {
	$moyenne = 0;
	$rendt{$joueur} = 0;
	$ecart_type{$joueur} = 0;
	$sharpe{$joueur} = 1000000;
    } else {
	# calcul de la moyenne des variations
	$moyenne = $cumuldelta{$joueur} / $ccount;
#    print "Joueur n" . $joueur . "\nMoyenne=" . $moyenne;
	$rendt{$joueur} = $portif{$joueur} - 100;
#    print "\tRsultat=" . $rendt{$joueur} . "\n";
	# calcul de la variance et de l'cart-type de la priode
	$variance = ($cumulcarres{$joueur} / $ccount) - ($moyenne * $moyenne);
	$ecart_type{$joueur} = sqrt($variance);
#    print "Variance=" . $variance . "\tEcart-type=" . $ecart_type{$joueur} . "\n";
	# calcul du ratio de Sharpe
	if ($ecart_type{$joueur} != 0) {
	    $sharpe{$joueur} = (($rendt{$joueur} / 100) - $livreta) / $ecart_type{$joueur};
	} else {
	    next;
	}
#    print "Sharpe=" . $sharpe{$joueur} . "\n";
    }
}
if ($ccount == 0) {
    #
    # tri par ordre alphabtique des pseudos (listes valides)
    #
    @classement = sort { $pseudo{$a} cmp $pseudo{$b} } keys %pseudo;
} else {
    #
    # tri par ordre dcroissant du rendement
    #
    @classement = sort { $rendt{$b} <=> $rendt{$a} } keys %rendt;
}
my ($sec,$min,$heure,$jour2,$mois2,$an2,$js,$ja,$hiv) = localtime(time);
$an2 += 1900;
$mois2++;
$mois2 = $mois2 < 10 ? "0" . $mois2 : $mois2;
$jour2 = $jour2 < 10 ? "0" . $jour2 : $jour2;
#my $jour2 = 04;
#my $mois2 = 11;
#my $an2 = 2004;
# affichage du classement (dans un fichier en html)
my $fichier_rendt = "rendt" . substr($an1,2,2) . $mois1 . $jour1 . ".html";
if ($ccount == 0) {
    $fichier_rendt = "listes2007.html";
}
open(RENDT, ">" . $fichier_rendt);
if ($ccount == 0) {
    print RENDT "<h3 class=t6>Participants au " . $jour2 . "/" . $mois2 . "/" . $an2 . "</h3>\n";
} else {
    print RENDT "<h3 class=t6>Classement gnral du " . $jour1 . "/" . $mois1 . "/" . $an1 . "</h3>\n";
}
print RENDT "<h5 class=t6>(Fait le " . $jour2 . "/" . $mois2 . "/" . $an2 ."  ";
print RENDT $heure . ":" . $min . ":" . $sec . ")</h5>\n";
print RENDT "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
$place = 0;
foreach $joueur (@classement) {
    if ($joueur > 1000) {
	print RENDT "<tr class=t1><td>&nbsp;</td>";
    } else {
	$place += 1;
	print RENDT "<tr class=t1><td>" . $place . "</td>";
    }
    print RENDT "<td colspan=2>" . $pseudo{$joueur} . "</td>";
    if ($rendt{$joueur} < 0) {
	print RENDT "<td colspan=2 class=t3><b>";
    } else {
	print RENDT "<td colspan=2 class=t2><b>";
    }
    print RENDT sprintf("%+.2f",$rendt{$joueur}) . "</b></td></tr>\n";
    my $impair = 1;
    $select_vallib->execute($joueur);
    # pour chaque ligne du portefeuille
    while (@vallib = $select_vallib->fetchrow_array()) {
	if ($impair) {
	    print RENDT "<TR CLASS=t7><TD>&nbsp;</TD>";
	    $impair = 0;
	} else {
	    print RENDT "<TR CLASS=t8><TD BGCOLOR=WHITE>&nbsp;</TD>";
	    $impair =1;
	}
	$valeur = $vallib[0];
	print RENDT "<TD>$vallib[1]</TD>";
	print RENDT "<TD>$cotedorigine{$valeur}</TD>";
	print RENDT "<TD>$cotedujour{$valeur}</TD>";
        $variation = 100 * ($cotedujour{$valeur} - $cotedorigine{$valeur}) / $cotedorigine{$valeur};
        $pourcentage = sprintf("%+01.2f%%",$variation);
        if ($variation >= 0) {
            print RENDT "<TD CLASS=t4>$pourcentage</TD></TR>\n";
        } else {
            print RENDT "<TD CLASS=t5>$pourcentage</TD></TR>\n";
        }
    }
    print RENDT "\n";
}
$select_vallib->finish;
print RENDT "</tbody></table>\n<br>\n";
close(RENDT);
#
# tri par ordre croissant de l'cart type
#
@classement = sort { $ecart_type{$a} <=> $ecart_type{$b} } keys %sharpe;
($sec,$min,$heure,$jour2,$mois2,$an2,$js,$ja,$hiv) = localtime(time);
$an2 += 1900;
$mois2++;
$mois2 = $mois2 < 10 ? "0" . $mois2 : $mois2;
$jour2 = $jour2 < 10 ? "0" . $jour2 : $jour2;
# affichage du classement (dans un fichier en html)
my $fichier_etype = "etype" . substr($an1,2,2) . $mois1 . $jour1 . ".html";
open(ECARTTYPE, ">" . $fichier_etype);
print ECARTTYPE "<h3 class=t6>Classement cart-type du " . $jour1 . "/" . $mois1 . "/" . $an1 . "</h3>\n";
print ECARTTYPE "<h5 class=t6>(Fait le " . $jour2 . "/" . $mois2 . "/" . $an2 . "  ";
print ECARTTYPE $heure . ":" . $min . ":" . $sec . ")</h5>\n";
print ECARTTYPE "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
$place = 0;
foreach $joueur (@classement) {
    if ($joueur > 1000) {
	print ECARTTYPE "<tr class=t1><td>&nbsp;</td>";
    } else {
	$place += 1;
	print ECARTTYPE "<tr class=t1><td>" . $place . "</td>";
    }
    print ECARTTYPE "<td colspan=2>" . $pseudo{$joueur} . "</td>";
    if ($sharpe{$joueur} < 0) {
	print ECARTTYPE "<td colspan=2 class=t3><b>";
    } else {
	print ECARTTYPE "<td colspan=2 class=t2><b>";
    }
    print ECARTTYPE sprintf("%+.5f",$ecart_type{$joueur}) . "</b></td></tr>\n";
}
print ECARTTYPE "</tbody></table>\n<br>\n";
close(ECARTTYPE);
#
# tri par ordre dcroissant du ratio de Sharpe
#
@classement = sort { $sharpe{$b} <=> $sharpe{$a} } keys %sharpe;
($sec,$min,$heure,$jour2,$mois2,$an2,$js,$ja,$hiv) = localtime(time);
$an2 += 1900;
$mois2++;
$mois2 = $mois2 < 10 ? "0" . $mois2 : $mois2;
$jour2 = $jour2 < 10 ? "0" . $jour2 : $jour2;
# affichage du classement (dans un fichier en html)
my $fichier_sharpe = "sharpe" . substr($an1,2,2) . $mois1 . $jour1 . ".html";
open(SHARPE, ">" . $fichier_sharpe);
print SHARPE "<h3 class=t6>Classement Sharpe du " . $jour1 . "/" . $mois1 . "/" . $an1 . "</h3>\n";
print SHARPE "<h5 class=t6>(Fait le " . $jour2 . "/" . $mois2 . "/" . $an2 . "  ";
print SHARPE $heure . ":" . $min . ":" . $sec . ")</h5>\n";
print SHARPE "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
$place = 0;
foreach $joueur (@classement) {
    if ($joueur > 1000) {
	print SHARPE "<tr class=t1><td>&nbsp;</td>";
    } else {
	$place += 1;
	print SHARPE "<tr class=t1><td>" . $place . "</td>";
    }
    print SHARPE "<td colspan=2>" . $pseudo{$joueur} . "</td>";
    if ($sharpe{$joueur} < 0) {
	print SHARPE "<td colspan=2 class=t3><b>";
    } else {
	print SHARPE "<td colspan=2 class=t2><b>";
    }
    print SHARPE sprintf("%+.2f",$sharpe{$joueur}) . "</b></td></tr>\n";
}
print SHARPE "</tbody></table>\n<br>\n";
close(SHARPE);
$select_resultat->finish;
$select_joueur->finish;
$select_cotes->finish;
$select_jours->finish;
#
# fin de la boucle, on ferme!
#
$dbh->disconnect;
#
# Transfert des fichiers html vers le serveur (option -f)
#
if ($opt_f) {
    my $ftp = new Net::FTP($urlftp, Debug => 0, Passive => 1);
    $ftp->login($loginftp,$mdpftp);
    $ftp->cwd("/thebull");
    $ftp->put($fichier_rendt);
    $ftp->put($fichier_etype);
    $ftp->put($fichier_sharpe);
    $ftp->quit;
}
# c'est fini!
exit(0);


