#!/usr/bin/perl -w
# valobull07.pl
# Auteur: Jean-Marie Pacquet
#
# il calcule la performance de chaque valeur du jeu  une date donne
# il classe ensuite les valeurs par ordre dcroissant de slections
# et calcule la moyenne (pondre ou non) de l'ensemble
# il classe aussi les valeurs par performance
#
use strict;
use Getopt::Std;
use DBI;
use Net::FTP;

#
###############################################################################
#
# date du classement
my $anc;
my $moisc;
my $jourc;
# Rcuprer les options de la ligne de commande
our($opt_d, $opt_f);
if (getopts('fd:')) {
    print $opt_d . "\n" if ($opt_d);
} else {
    die "valobull: usage: valobull [-d aaaa-mm-jj] [-f]\n";
}
if ($opt_d) {
    $anc = substr($opt_d,0,4);
    $moisc = substr($opt_d,5,2);
    $jourc = substr($opt_d,8,2);
} else {
    $anc = 2007;
    $moisc = 12;
    $jourc = 31;
}
# Ouvrir la base de donnes
#
my $dbh = DBI->connect('DBI:Pg:dbname=thebull',
		       'utilisateur-sql',
		       'mot-de-passe-sql',
		       {
			   AutoCommit => 0
		       }
		       ) or die "Erreur de connexion  la base: " . DBI->errstr;
# date de rfrence pour les cotes
my $origine = "2006-12-31";
# date de dernire cotation
my $an1 = "2007";
my $mois1 = "01";
my $jour1 = "01";
# compteur pour les jours de cotation
my $ccount; 
#
# prparer la liste des jours de cotation depuis l'origine
#
my $select_jours = $dbh->prepare('SELECT DISTINCT jour FROM cotations07 WHERE jour > ?');
my $date; # rsultat de la requte $select_jours
#
# prparer la liste des cotations  une date donne
#
my $select_cotes = $dbh->prepare('SELECT cote,valeur FROM cotations07 WHERE jour = ?');
my $select_dividende = $dbh->prepare('SELECT dividende,valeur FROM dividendes07 WHERE jour = ?');
my @cote; # tableau de rsultat de la requte $select_cotes
my %cotedorigine; # tableau des cotes d'origine index par valeur
my %cotedujour; # tableau des cotes du jour index par valeur
my %dividende; # tableau des dividendes index par valeur
my @detachement; # tableau de rsultat de la requte $select_dividende
my %performance; # tableau des performances index par valeur
my @classement; # le classement final
my $valeur;
#
# prparer la liste des valeurs trie par nombre de slections et libell
#
my $select_valeurs = $dbh->prepare('SELECT DISTINCT libelle, COUNT(1) AS "Nombre de slections", lignes07.valeur FROM lignes07
 JOIN valeurs07 ON valeurs07.valeur = lignes07.valeur
  WHERE valeurs07.valeur != 485 and valeurs07.valeur != 486 
  GROUP BY lignes07.valeur, valeurs07.libelle
   ORDER BY "Nombre de slections" DESC');
my @preference; # rsultat de la requte $select_valeurs
#
# prparer la recherche d'un libell
#
my $select_libelle = $dbh->prepare('SELECT libelle FROM valeurs07 WHERE valeur = ?');
my $libelle; # rsultat de la recherche d'un libell

die "Impossible de prparer les requtes\n"
    unless defined $select_jours && defined $select_dividende
    && defined $select_cotes && defined $select_valeurs
    && defined $select_libelle;
#
# initialisation des cotes des valeurs
#
$select_cotes->execute($origine);
$ccount = 0;
while (@cote = $select_cotes->fetchrow_array()) {
    $cotedorigine{$cote[1]} = $cote[0];
    $cotedujour{$cote[1]} = $cote[0];
    $dividende{$cote[1]} = 0;
    $ccount++;
}
if ($ccount > 0) {
    print $ccount . " initialisations de cotes  la date " . $origine . "\n";
}
#
# boucle de mise  jour des cotations
#
my $somme;
my $numligne;
my $variation;
my $pourcentage;
my $resultat;
$ccount = 0; # nombre de jours de cotation
$select_jours->execute($origine);
while ($date = $select_jours->fetchrow_array()) {
    my $ann = substr($date,0,4);
    my $moisn = substr($date,5,2);
    my $journ = substr($date,8,2);
    if (($anc < $ann) or (($anc == $ann) and ($moisc < $moisn))
	or (($anc == $ann) and ($moisc == $moisn) and ($jourc < $journ))) {
	last;
    }
    # dernire date de cotation
    $an1 = $ann;
    $mois1 = $moisn;
    $jour1 = $journ;
    # mise  jour des dividendes (pour les valeurs dont c'est le jour de dtachement)
    $select_dividende->execute($date);
    while (@detachement = $select_dividende->fetchrow_array()) {
        $dividende{$detachement[1]} += $detachement[0];
    }
    # mise  jour des cotes (pour les valeurs qui ont cot)
    $select_cotes->execute($date);
    while (@cote = $select_cotes->fetchrow_array()) {
	$cotedujour{$cote[1]} = $cote[0];
	if ($dividende{$cote[1]}) {
	    $cotedujour{$cote[1]} = sprintf("%.2f",$cote[0] + $dividende{$cote[1]});
	}
    }
    # ajustement du nombre de jours de cote
    $ccount += 1;
}
print $ccount . " jours de cote\n";
$select_cotes->finish;
$select_jours->finish;
#
# boucle finale
#
my $moyenne;
my $cumul1;
my $cumul2;
my $ligne;
my $poids;
my $nombre;
my $rendt;
my ($sec,$min,$heure,$jour2,$mois2,$an2,$js,$ja,$hiv) = localtime(time);
$an2 += 1900;
$mois2++;
$mois2 = $mois2 < 10 ? "0" . $mois2 : $mois2;
$jour2 = $jour2 < 10 ? "0" . $jour2 : $jour2;
# affichage de la liste (dans un fichier en html)
$select_valeurs->execute();
if ($select_valeurs->rows == 0) {
    die "Pas de valeurs\n";
}
my $fichier_pref = "pref07" . substr($an1,2,2) . $mois1 . $jour1 . ".html";
open(PREF, ">" . $fichier_pref);
print PREF "<h3 class=t6>La performance des prfres au " . $jour1 . "/" . $mois1 . "/" . $an1 . "</h3>\n";
print PREF "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
$cumul1 = 0;
$cumul2 = 0;
$ligne = 0;
$nombre = 0;
while (@preference = $select_valeurs->fetchrow_array()) {
    print PREF "<tr class=t1><td align=right>" . $preference[1] . "</td>";
    print PREF "<td>" . $preference[0] . "</td>";
    $ligne++;
    $poids = $preference[1];
    $rendt = 100 * ($cotedujour{$preference[2]} - $cotedorigine{$preference[2]}) / $cotedorigine{$preference[2]};
    $performance{$preference[2]} = $rendt;
    if ($rendt < 0) {
	print PREF "<td class=t3>" . sprintf("%+.2f",$rendt) . "</td>";
    } else {
	print PREF "<td class=t2>" . sprintf("%+.2f",$rendt) . "</td>";
    }
    $cumul1 += $rendt;
    $moyenne = $cumul1 / $ligne;
    if ($moyenne < 0) {
	print PREF "<td class=t3>" . sprintf("%+.2f",$moyenne) . "</td>";
    } else {
	print PREF "<td class=t2>" . sprintf("%+.2f",$moyenne) . "</td>";
    }
    $cumul2 += ($rendt * $poids);
    $nombre += $poids;
    $moyenne = $cumul2 / $nombre;
    if ($moyenne < 0) {
	print PREF "<td class=t3>" . sprintf("%+.2f",$moyenne) . "</td>";
    } else {
	print PREF "<td class=t2>" . sprintf("%+.2f",$moyenne) . "</td>";
    }
    print PREF "</td></tr>\n";
}
$select_valeurs->finish;
print PREF "</tbody></table>\n<br>\n";
close(PREF);
#
# tri par ordre dcroissant de performance
#
@classement = sort { $performance{$b} <=> $performance{$a} } keys %performance;
# affichage du classement (dans un fichier en html)
my $fichier_perf = "perf07" . substr($an1,2,2) . $mois1 . $jour1 . ".html";
open(PERF, ">" . $fichier_perf);
print PERF "<h3 class=t6>Les meilleures au " . $jour1 . "/" . $mois1 . "/" . $an1 . "</h3>\n";
print PERF "<h5 class=t6>(Fait le " . $jour2 . "/" . $mois2 . "/" . $an2 ."  ";
print PERF $heure . ":" . $min . ":" . $sec . ")</h5>\n";
print PERF "<table border=0 cellpadding=1 cellspacing=1 width=100%><tbody>";
my $place = 0;
my $impair = 0;
foreach $valeur (@classement) {
    $place += 1;
    if ($impair) {
	print PERF "<TR CLASS=t7><TD>&nbsp;</TD>";
	$impair = 0;
    } else {
	print PERF "<TR CLASS=t8><TD BGCOLOR=WHITE>&nbsp;</TD>";
	$impair = 1;
    }
    $select_libelle->execute($valeur);
    if ($libelle = $select_libelle->fetchrow_array()) {
	print PERF "<TD>$place</TD>";
	print PERF "<TD>$libelle</TD>";
	print PERF "<TD>$cotedorigine{$valeur}</TD>";
	print PERF "<TD>$cotedujour{$valeur}</TD>";
        $variation = 100 * ($cotedujour{$valeur} - $cotedorigine{$valeur}) / $cotedorigine{$valeur};
        $pourcentage = sprintf("%+01.2f%%",$variation);
        if ($variation >= 0) {
            print PERF "<TD CLASS=t4>$pourcentage</TD>";
        } else {
            print PERF "<TD CLASS=t5>$pourcentage</TD>";
        }
    }
    print PERF "</TR>\n";
}
print PERF "</tbody></table>\n<br>\n";
close(PERF);
$select_libelle->finish;
#
# fin de la boucle, on ferme!
#
$dbh->disconnect;
#
# Transfert du fichier html vers le serveur (option -f)
#
if ($opt_f) {
    my $ftp = new Net::FTP("serveur-perso-ftp", Debug => 0, Passive => 1);
    $ftp->login("utilisateur-ftp","mot-de-passe-ftp");
    $ftp->cwd("/thebull");
    $ftp->put($fichier_pref);
    $ftp->put($fichier_perf);
    $ftp->quit;
}
# c'est fini!
exit(0);


